from django.conf.urls import url


urlpatterns = [
    url(r'^$', 'tsuru_autoscale.datasource.views.list', name='datasource-list'),
    url(r'^new/$', 'tsuru_autoscale.datasource.views.new', name='datasource-new'),
    url(r'^(?P<name>[\w\s-]+)/remove/$', 'tsuru_autoscale.datasource.views.remove', name='datasource-remove'),
    url(r'^(?P<name>[\w\s-]+)/$', 'tsuru_autoscale.datasource.views.get', name='datasource-get'),
]



from django.shortcuts import render

from tsuru_autoscale.event import client


def list(request, alarm_name):
    token = request.GET.get("TSURU_TOKEN")
    events = client.list(alarm_name, token).json()
    context = {
        "list": events,
    }
    return render(request, "event/list.html", context)



from django.conf.urls import url


urlpatterns = [
    url(r'^(?P<instance>[\w-]+)/$', 'tsuru_autoscale.wizard.views.new', name='wizard-new'),
    url(r'^(?P<instance>[\w-]+)/remove/$', 'tsuru_autoscale.wizard.views.remove', name='wizard-remove'),
    url(r'^$', 'tsuru_autoscale.wizard.views.new', name='wizard-new'),
]



from django.conf.urls import include, url


urlpatterns = [
    url(r'^instance/$', 'tsuru_autoscale.instance.views.list', name='instance-list'),
    url(r'^datasource/', include('tsuru_autoscale.datasource.urls')),
    url(r'^alarm/', include('tsuru_autoscale.alarm.urls')),
    url(r'^action/', include('tsuru_autoscale.action.urls')),
    url(r'^instance/', include('tsuru_autoscale.instance.urls')),
    url(r'^wizard/', include('tsuru_autoscale.wizard.urls')),
    url(r'^', include('tsuru_autoscale.app.urls')),
]



from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.contrib import messages

from tsuru_autoscale.alarm.forms import AlarmForm, datasource_list, action_list, service_instance_list
from tsuru_autoscale.alarm import client


def new(request):
    token = request.GET.get("TSURU_TOKEN")

    form = AlarmForm(request.POST or None)
    form.fields['datasource'].choices = datasource_list(token)
    form.fields['actions'].choices = action_list(token)
    form.fields['instance'].choices = service_instance_list(token)

    if form.is_valid():
        client.new(form.cleaned_data, token)
        messages.success(request, u"Alarm saved.")
        url = "{}?TSURU_TOKEN={}".format(reverse("alarm-list"), token)
        return redirect(url)

    context = {"form": form}
    return render(request, "alarm/new.html", context)


def list(request):
    token = request.GET.get("TSURU_TOKEN")
    alarms = client.list(token).json()
    context = {
        "list": alarms,
    }
    return render(request, "alarm/list.html", context)


def remove(request, name):
    token = request.GET.get("TSURU_TOKEN")
    client.remove(name, token)
    messages.success(request, u"Alarm {} removed.".format(name))
    url = "{}?TSURU_TOKEN={}".format(reverse('alarm-list'), token)
    return redirect(url)


def get(request, name):
    token = request.GET.get("TSURU_TOKEN")
    alarm = client.get(name, token).json()
    context = {
        "item": alarm,
    }
    return render(request, "alarm/get.html", context)



from django.conf.urls import url


urlpatterns = [
    url(r'^$', 'tsuru_autoscale.alarm.views.list', name='alarm-list'),
    url(r'^new/$', 'tsuru_autoscale.alarm.views.new', name='alarm-new'),
    url(r'^(?P<name>[\w\s-]+)/remove/$', 'tsuru_autoscale.alarm.views.remove', name='alarm-remove'),
    url(r'^(?P<alarm_name>[\w\s-]+)/event/$', 'tsuru_autoscale.event.views.list', name='event-list'),
    url(r'^(?P<name>[\w\s-]+)/$', 'tsuru_autoscale.alarm.views.get', name='alarm-get'),
]



from django.test import TestCase
from django.core.urlresolvers import reverse

from tsuru_autoscale.alarm.forms import AlarmForm
from tsuru_autoscale.alarm import client

import httpretty
import mock

import os


class RemoveTestCase(TestCase):
    @mock.patch("tsuru_autoscale.alarm.client.list")
    @mock.patch("tsuru_autoscale.alarm.client.remove")
    def test_remove(self, remove_mock, list_mock):
        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-remove", args=["name"]))
        response = self.client.delete(url)

        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-list"))
        self.assertRedirects(response, url)
        remove_mock.assert_called_with("name", "bla")


class NewTestCase(TestCase):
    @mock.patch("tsuru_autoscale.alarm.client.service_instance_list")
    @mock.patch("tsuru_autoscale.datasource.client")
    @mock.patch("tsuru_autoscale.action.client")
    def test_new(self, ds_client_mock, a_client_mock, sil):
        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-new"))
        response = self.client.get(url)
        self.assertTemplateUsed(response, "alarm/new.html")
        self.assertIsInstance(response.context['form'], AlarmForm)
        self.assertFalse(response.context['form'].is_bound)

    @mock.patch("tsuru_autoscale.alarm.client.service_instance_list")
    @mock.patch("tsuru_autoscale.datasource.client")
    @mock.patch("tsuru_autoscale.action.client")
    def test_new_invalid_post(self, ds_client_mock, a_client_mock, sli):
        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-new"))
        response = self.client.post(url, {})
        self.assertFalse(response.context['form'].is_valid())

    @mock.patch("tsuru_autoscale.alarm.client.service_instance_list")
    @mock.patch("tsuru_autoscale.action.client")
    @mock.patch("tsuru_autoscale.datasource.client")
    @mock.patch("tsuru_autoscale.alarm.client.list")
    @mock.patch("tsuru_autoscale.alarm.client.new")
    def test_new_post(self, new_mock, list_mock, ds_client_mock, a_client_mock, sil):
        json_mock = mock.Mock()
        json_mock.json.return_value = [{"Name": "bla"}]
        sil.return_value = json_mock

        json_mock = mock.Mock()
        json_mock.json.return_value = [{"Name": "bla"}]
        a_client_mock.list.return_value = json_mock

        json_mock = mock.Mock()
        json_mock.json.return_value = [{"Name": "bla"}]
        ds_client_mock.list.return_value = json_mock
        data = {
            'name': u'name',
            'expression': u'x > 10',
            'enabled': True,
            'wait': 10,
            'datasource': 'bla',
            'actions': ['bla'],
            'instance': 'bla',
        }

        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-new"))
        response = self.client.post(url, data)

        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-list"))
        self.assertRedirects(response, url)
        new_mock.assert_called_with(data, "bla")


class ListTestCase(TestCase):
    @mock.patch("tsuru_autoscale.alarm.client.list")
    def test_list(self, list_mock):
        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-list"))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "alarm/list.html")
        self.assertIn('list', response.context)
        list_mock.assert_called_with("bla")


class GetTestCase(TestCase):
    @mock.patch("tsuru_autoscale.alarm.client.get")
    def test_get(self, get_mock):
        result_mock = mock.Mock()
        result_mock.json.return_value = {"name": "ble"}
        get_mock.return_value = result_mock

        url = "{}?TSURU_TOKEN=bla".format(reverse("alarm-get", args=["ble"]))
        response = self.client.get(url)

        self.assertTemplateUsed(response, "alarm/get.html")
        self.assertIn('item', response.context)
        get_mock.assert_called_with("ble", "bla")


class ClientTestCase(TestCase):
    def setUp(self):
        httpretty.enable()

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_new(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.POST,
            "http://autoscalehost.com/alarm",
        )

        client.new({}, "token")

    def test_list(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/alarm",
        )

        client.list("token")

    def test_remove(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.DELETE,
            "http://autoscalehost.com/alarm/name",
        )

        client.remove("name", "token")

    def test_get(self):
        os.environ["AUTOSCALE_HOST"] = "http://autoscalehost.com"
        httpretty.register_uri(
            httpretty.GET,
            "http://autoscalehost.com/alarm/name",
            "result",
        )

        result = client.get("name", "token")
        self.assertEqual(result.text, "result")


class AlarmFormTestCase(TestCase):
    def test_required_fields(self):
        fields = {
            "name": True,
            "expression": True,
            "enabled": True,
            "wait": True,
        }

        form = AlarmForm()

        for field, required in fields.items():
            self.assertEqual(form.fields[field].required, required)



from django.shortcuts import render

from tsuru_autoscale.instance import client
from tsuru_autoscale.wizard import client as wclient

import urllib


def index(request, app):
    token = request.session.get('tsuru_token').split(" ")[-1]
    instances = client.list(token).json() or []

    instance = None
    auto_scale = None
    events = None

    for inst in instances:
        if app in inst.get('Apps', []):
            instance = inst

            response = wclient.get(instance["Name"], token)
            if response.status_code == 200:
                auto_scale = response.json()
                events = wclient.events(instance["Name"], token).json()
    context = {
        "instance": instance,
        "auto_scale": auto_scale,
        "token": urllib.quote(token),
        "app": app,
        "events": events,
    }
    return render(request, "app/index.html", context)
